/*
 * This script counts selected (if none selected: all) tracks and
 * prints a summary in the user window.
 *
 * Copyright smf, some ideas taken from the QCad example scripts
 * last change: 2014-12-13
 */

include("../QCTrack.js");

function TrackList(guiAction) {
    EAction.call(this, guiAction);
}

TrackList.prototype = new EAction();
TrackList.includeBasePath = includeBasePath;

TrackList.prototype.beginEvent = function() {
    var document = EAction.getDocument();

    var dialog = WidgetFactory.createWidget(
                   "scripts/QCTrack/TrackList",
                   "TrackListDialog.ui");
    var widgets = getWidgets(dialog);
    var checkIds = document.querySelectedEntities();
    if (checkIds.length == 0) {
      widgets["entAllBlocks"].checked = true;
      widgets["groupEnt"].enabled = false;
    }
    if (!dialog.exec()) { //the user aborted
        this.terminate();
        return;
    }
    var useNumbersInsideBlock = widgets["nameNumber"].checked;
    var evalCost = widgets["nameBlockWithCost"].checked;
    if (widgets["entAllBlocks"].checked) {
      checkIds = document.queryAllBlockReferences();
    }
    var list = new Array(); //each element contains a three-field array containing the track number and the current amount and the current cost of this track
    var blockCount = 0, nmbrCount = 0;

    for (var i = 0; i < checkIds.length; i++) { //first run: count all blocks with TR_NMBRS
      var id = checkIds[i];
      var block = document.queryEntity(id);
      if (!isBlockReferenceEntity(block)) {
        continue;
      }
      blockCount++;
      if (useNumbersInsideBlock) {
        var data = block.getData();
        var subIds = document.queryBlockEntities(data.getReferencedBlockId());
        for (var j = 0; j < subIds.length; j++) {
          var text = data.queryEntity(subIds[j]);
          if (isTextEntity(text) && text.getLayerName() == "TR_NMBRS") {
            var nmbr = text.getPlainText();
            if (nmbr == undefined || nmbr == "") {
              continue;
            }
            nmbrCount++;
            inc(list, nmbr, 1, 0);
          }
        }
      }
      else {
        var blockName = block.getReferencedBlockName();
        inc(list, blockName, 1, 0);
        if (evalCost) {
          var data = block.getData();
          var subIds = document.queryBlockEntities(data.getReferencedBlockId());
          for (var j = 0; j < subIds.length; j++) {
            var text = data.queryEntity(subIds[j]);
            if (isTextEntity(text) && text.getLayerName() == "TR_COST") {
              var addCost = parseInt(text.getPlainText());
              if (addCost > 0) {
                inc(list, blockName, 0, addCost);
              }
            }
          }
        }
      }
    }
    if (useNumbersInsideBlock) {
      EAction.handleUserMessage("processed " + blockCount + " blocks containing " + nmbrCount + " numbers:");
    }
    else {
      EAction.handleUserMessage("processed " + blockCount + " blocks:");
    }
    list.sort(function(a, b) {
      return a[0].localeCompare(b[0]);
    });
    for (var i = 0; i < list.length; i++) { //second run: print track numbers and track count
      if (evalCost) {
        EAction.handleUserMessage(list[i][0] + ": " + list[i][1] + ": cost: " + list[i][2]);
      }
      else {
        EAction.handleUserMessage(list[i][0] + ": " + list[i][1]);
      }
    }

    this.terminate();
};

function inc(list, name, by1, by2) {
  var k = 0;
  while (k < list.length) {
    if (list[k][0] == name) {
      list[k][1] += by1;
      list[k][2] += by2;
      break;
    }
    k++;
  }
  if (k == list.length) {
    list.push([name, by1, by2]);
  }
}

TrackList.init = function(basePath) {
    var action = new RGuiAction(qsTr("&List tracks"), RMainWindowQt.getMainWindow());
    action.setRequiresDocument(true);
    action.setScriptFile(basePath + "/TrackList.js");
    action.setIcon(basePath + "/TrackList.svg");
    action.setDefaultShortcut(new QKeySequence("q,l"));
    action.setSortOrder(504);
    EAction.addGuiActionTo(action, QCTrack, true, false, false);
    var appWin = EAction.getMainWindow();
    appWin.addSelectionListener(action);
};
